@@ -1,5 +1,5 @@
 from pathlib import Path
-from typing import Callable, cast
+from typing import cast
 
 from recon.corpus import Corpus
 from recon.stats import (
@@ -7,7 +7,7 @@ from recon.stats import (
     calculate_label_distribution_similarity,
     get_ner_stats,
 )
-from recon.types import Example, NERStats
+from recon.types import NERStats
 from wasabi import Printer
 
 
@@ -28,7 +28,10 @@ def stats(data_dir: Path) -> None:
             msg.text(f"Stats for {ds.capitalize()} data")
             msg.text("--------------------")
             msg.table(
-                {"N Examples": ner_stats.n_examples, "N Annotations": ner_stats.n_annotations,}
+                {
+                    "N Examples": ner_stats.n_examples,
+                    "N Annotations": ner_stats.n_annotations,
+                }
             )
             msg.info(f"Labels in {ds}")
             msg.text(sorted_labels)
