@@ -1,26 +1,22 @@
 """Make corrections to your data."""
 
-import copy
-from collections import defaultdict
-from typing import Any, DefaultDict, Dict, List, cast
+from typing import Any, Dict, List, cast
 
-import spacy
-from spacy.tokens import Doc as SpacyDoc, Span as SpacySpan
+from spacy.tokens import Span as SpacySpan
 from wasabi import msg
 
 from .operations import operation
-from .preprocess import SpacyPreProcessor
-from .types import Correction, Example, Span, Token, TransformationCallbacks
+from .types import Correction, Example, Span, Token
 
 
 @operation("recon.v1.rename_labels")
 def rename_labels(example: Example, label_map: Dict[str, str]) -> Example:
     """Rename labels in a copy of List[Example] data
-    
+
     Args:
         example (Example): Input Example
         label_map (Dict[str, str]): One-to-one mapping of label names
-    
+
     Returns:
         Example: Copy of Example with renamed labels
     """
@@ -37,18 +33,18 @@ def fix_annotations(
     dryrun: bool = False,
 ) -> Example:
     """Fix annotations in a copy of List[Example] data.
-    
+
     This function will NOT add annotations to your data.
     It will only remove erroneous annotations and fix the
     labels for specific spans.
-    
+
     Args:
         example (Example): Input Example
         corrections (Dict[str, str]): Dictionary of corrections mapping entity text to a new label.
             If the value is set to None, the annotation will be removed
         case_sensitive (bool, optional): Consider case of text for each correction
         dryrun (bool, optional): Treat corrections as a dryrun and just print all changes to be made
-    
+
     Returns:
         Example: Example with fixed annotations
     """
@@ -96,7 +92,7 @@ def fix_annotations(
 
 def corrections_from_dict(corrections_dict: Dict[str, Any]) -> List[Correction]:
     """Create a list of Correction objects from a simpler config for
-    corrections using a Dict representation mapping keys to either the label to 
+    corrections using a Dict representation mapping keys to either the label to
     convert to or a tuple of (from_label, to_label) pairings or (List[from_labels], to_label)
     pairings if you want to convert as subset of labels at a time
 
@@ -133,26 +129,35 @@ def corrections_from_dict(corrections_dict: Dict[str, Any]) -> List[Correction]:
     return corrections
 
 
-@operation("recon.v1.strip_annotations")
+@operation("recon.v1.strip_annotations", pre=["recon.v1.spacy"], handles_tokens=False)
 def strip_annotations(
-    example: Example, strip_chars: List[str] = [".", "!", "?", "-", ":", " "]
+    example: Example,
+    strip_chars: List[str] = [".", "!", "?", "-", ":", " "],
+    preprocessed_outputs: Dict[str, Any] = {},
 ) -> Example:
     """Strip punctuation and spaces from start and end of annotations.
     These characters are almost always a mistake and will confuse a model
-    
+
     Args:
         example (Example): Input Example
         strip_chars (List[str], optional): Characters to strip.
-    
+
     Returns:
         Example: Example with stripped spans
     """
 
+    preprocessed_outputs["recon.v1.spacy"]
+
+    def fix_tokens(example: Example, span: Span) -> bool:
+        fix_tokens = example.tokens and any(example.tokens)
+        return bool(
+            fix_tokens and span.token_start and span.token_end and span.token_start < span.token_end
+        )
+
     for s in example.spans:
         for ch in strip_chars:
             if s.text.startswith(ch):
                 ch = s.text[0]
-
                 while ch in strip_chars:
                     s.text = s.text[1:]
                     s.start += 1
@@ -166,15 +171,10 @@ def strip_annotations(
     return example
 
 
-nlp = spacy.blank("en")
-nlp.add_pipe(nlp.create_pipe("sentencizer"))
-spacy_pre = SpacyPreProcessor(nlp)
-
-
-@operation("recon.v1.split_sentences", pre=[spacy_pre])
+@operation("recon.v1.split_sentences", pre=["recon.v1.spacy"])
 def split_sentences(example: Example, preprocessed_outputs: Dict[str, Any] = {}) -> List[Example]:
-    """Split a single example into multiple examples by splitting the text into 
-    multiple sentences and resetting entity and token offsets based on offsets 
+    """Split a single example into multiple examples by splitting the text into
+    multiple sentences and resetting entity and token offsets based on offsets
     relative to sentence boundaries
 
     Args:
@@ -182,7 +182,7 @@ def split_sentences(example: Example, preprocessed_outputs: Dict[str, Any] = {})
         preprocessed_outputs (Dict[str, Any], optional): Outputs of preprocessors.
 
     Returns:
-        List[Example]: List of split examples. 
+        List[Example]: List of split examples.
             Could be list of 1 if the example is just one sentence.
     """
     doc = preprocessed_outputs["recon.v1.spacy"]
@@ -206,7 +206,7 @@ def split_sentences(example: Example, preprocessed_outputs: Dict[str, Any] = {})
     for sent in doc.sents:
         sent_doc = sent.as_doc()
         new_example = Example(
-            text=sent_doc.text,
+            text=sent.text,
             spans=[
                 Span(
                     text=e.text,
