@@ -2,23 +2,20 @@
 the [Prodigy](https://prodi.gy) format.
 """
 
-import functools
 from pathlib import Path
-from typing import Any, Dict, List, Union
+from typing import Any, Dict, List
 
 import srsly
 
-from .operations import registry
-from .registry import loading_pipelines
-from .types import Example, OperationState, OperationStatus, Transformation
+from .types import Example
 
 
 def read_jsonl(path: Path) -> List[Example]:
     """Read annotations in JSONL file format
-    
+
     Args:
         path (Path): Path to data
-    
+
     Returns:
         List[Example]: List of examples
     """
@@ -29,10 +26,10 @@ def read_jsonl(path: Path) -> List[Example]:
 
 def read_json(path: Path) -> List[Example]:
     """Read annotations in JSON file format
-    
+
     Args:
         path (Path): Path to data
-    
+
     Returns:
         List[Example]: List of examples
     """
@@ -43,10 +40,10 @@ def read_json(path: Path) -> List[Example]:
 
 def json_to_examples(data: List[Dict[str, Any]]) -> List[Example]:
     """Convert List of Dicts to List of typed Examples
-    
+
     Args:
         data (List[Dict[str, Any]]): Input List of Dicts to convert
-    
+
     Returns:
         List[Example]: List of typed Examples
     """
