@@ -1,11 +1,9 @@
-from typing import Any, Callable, Dict, List
+from typing import List
 
 import catalogue
 import spacy
 from spacy.language import Language
 
-from .types import Example
-
 loading_pipelines = catalogue.create("recon", "loading_pipelines", entry_points=True)
 tokenizers = catalogue.create("recon", "tokenizers", entry_points=True)
 
