@@ -2,9 +2,9 @@ from datetime import datetime
 from enum import Enum
 from typing import Any, Callable, Dict, List, NamedTuple, Optional, Tuple, cast
 
-from pydantic import BaseModel, Field, Schema, root_validator
+from pydantic import BaseModel, Extra, root_validator
 
-from .hashing import example_hash, span_hash, token_hash, tokenized_example_hash
+from .hashing import span_hash, token_hash, tokenized_example_hash
 
 
 class Span(BaseModel):
@@ -16,6 +16,7 @@ class Span(BaseModel):
     label: str
     token_start: Optional[int]
     token_end: Optional[int]
+    kb_id: Optional[str]
 
     def __hash__(self) -> int:
         return cast(int, span_hash(self))
@@ -42,6 +43,9 @@ class Example(BaseModel):
     meta: Dict[str, Any] = {}
     formatted: bool = False
 
+    class Config:
+        extra = Extra.allow
+
     @root_validator(pre=True)
     def span_text_must_exist(cls, values: Dict[str, Any]) -> Dict[str, Any]:
         if not values.get("formatted", False):
@@ -67,6 +71,25 @@ class Example(BaseModel):
     def __hash__(self) -> int:
         return cast(int, tokenized_example_hash(self))
 
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, Example):
+            return self.dict() == other.dict()
+        return False
+
+    def dict(self, **kwargs: Any) -> Dict:
+        res = super().dict(**kwargs)
+        keys = list(res.keys())
+        for k in keys:
+            if k not in self.schema()["properties"].keys():
+                del res[k]
+        return res
+
+
+class Entity(BaseModel):
+    id: Optional[str] = None
+    name: str
+    aliases: List[str]
+
 
 class TransformationType(str, Enum):
     EXAMPLE_ADDED = "EXAMPLE_ADDED"
@@ -128,6 +151,16 @@ class DatasetOperationsState(BaseModel):
     operations: List[OperationState]
 
 
+# class DatasetMeta(BaseModel):
+#     name: str
+#     version: str
+
+
+class CorpusMeta(BaseModel):
+    name: str
+    # versions: List[DatasetMeta]
+
+
 class OperationResult(BaseModel):
     data: Any
     state: OperationState
@@ -151,8 +184,8 @@ class AnnotationCount(BaseModel):
 
 class PredictionErrorExamplePair(BaseModel):
     """Dataclass representation of original Example in a PredictionError
-    vs the predicted Example from an EntityRecognizer 
-    
+    vs the predicted Example from an EntityRecognizer
+
     Attributes:
         original (Example): Original Example
         spans (List[Span]): List of entity spans
@@ -165,7 +198,7 @@ class PredictionErrorExamplePair(BaseModel):
 
 class PredictionError(BaseModel):
     """Representation of errors an EntityRecognizer makes on a labeled dataset.
-    
+
     Attributes:
         text (str): Span text with error
         true_label (str): True label in annotated Example
@@ -195,7 +228,7 @@ class HardestExample(BaseModel):
 class LabelDisparity(BaseModel):
     """Container for the number of disparities in a Dataset
     where some text is tagged as label1 in some places and label2 in others
-    
+
     Attributes:
         label1 (str): Label1
         label2 (str): Label2
@@ -221,7 +254,7 @@ class NERStats(BaseModel):
 
 class EntityCoverage(BaseModel):
     """Container for tracking how well an Entity is covered.
-    
+
     Attributes:
         text (str): The entity text
         label (str): The entity label
