@@ -1,20 +1,16 @@
-from collections import defaultdict
-from copy import deepcopy
-from typing import Any, Dict, List, Set, Tuple
-
-from spacy.language import Language
+from typing import List
 
 from .operations import operation
-from .types import Example, Span, TransformationCallbacks
+from .types import Example, Span
 
 
 @operation("recon.v1.upcase_labels")
 def upcase_labels(example: Example) -> Example:
     """Convert all span labels to uppercase to normalize
-    
+
     Args:
         example (Example): Input Example
-    
+
     Returns:
         Example: Example with fixed labels
     """
@@ -26,10 +22,10 @@ def upcase_labels(example: Example) -> Example:
 @operation("recon.v1.filter_overlaps")
 def filter_overlaps(example: Example) -> Example:
     """Filter overlapping entity spans by picking the longest one.
-    
+
     Args:
         example (Example): Input Example
-    
+
     Returns:
         List[Example]: Example with fixed overlaps
     """
