@@ -2,5 +2,5 @@
 set -x
 
 # Sort imports one per line, so autoflake can remove unused imports
-isort --recursive  --force-single-line-imports --thirdparty recon --apply recon tests docs/src
+isort recon tests docs/src
 sh ./scripts/format.sh
