@@ -1,11 +1,7 @@
-import shutil
-from pathlib import Path
 from typing import cast
 
 import pytest
-import recon
 from recon.dataset import Dataset
-from recon.operations import registry
 from recon.stats import get_ner_stats
 from recon.store import ExampleStore
 from recon.types import NERStats, OperationStatus, TransformationType
@@ -89,24 +85,65 @@ def test_apply_(example_data):
         assert t.type == TransformationType.EXAMPLE_CHANGED
 
 
+def test_rollback(example_data):
+    train_dataset = Dataset("train", example_data["train"])
+    ner_stats_pre: NERStats = cast(NERStats, train_dataset.apply(get_ner_stats))
+
+    assert len(train_dataset.operations) == 0
+
+    train_dataset.apply_("recon.v1.upcase_labels")
+
+    ner_stats_post: NERStats = cast(NERStats, train_dataset.apply(get_ner_stats))
+
+    pre_keys = sorted(ner_stats_pre.n_annotations_per_type.keys())
+    post_keys = sorted(ner_stats_post.n_annotations_per_type.keys())
+
+    assert pre_keys != post_keys
+
+    assert pre_keys == ["JOB_ROLE", "PRODUCT", "SKILL", "product", "skill"]
+    assert post_keys == ["JOB_ROLE", "PRODUCT", "SKILL"]
+
+    assert len(train_dataset.operations) == 1
+
+    train_dataset.rollback()
+
+    assert len(train_dataset.operations) == 0
+
+    ner_stats_rolled_back: NERStats = cast(NERStats, train_dataset.apply(get_ner_stats))
+    rolled_back_keys = sorted(ner_stats_rolled_back.n_annotations_per_type.keys())
+
+    assert pre_keys == rolled_back_keys
+
+
+def test_dataset_search(example_data):
+    train_dataset = Dataset("train", example_data["train"])
+
+    assert len(train_dataset.search("kotlin")) == 0
+    assert len(train_dataset.search("Kotlin")) == 1
+    assert len(train_dataset.search("kotlin", case_sensitive=False)) == 1
+    assert len(train_dataset.search("Software")) == 2
+    assert len(train_dataset.search("Software", case_sensitive=False)) == 4
+
+
 def test_dataset_to_from_disk(example_data, tmp_path):
+
     train_dataset = Dataset("train", example_data["train"])
     ner_stats_pre: NERStats = cast(NERStats, train_dataset.apply(get_ner_stats))
 
     assert len(train_dataset.operations) == 0
 
     with pytest.raises(FileNotFoundError):
-        train_dataset.to_disk(tmp_path / "train.jsonl")
+        train_dataset.to_disk(tmp_path)
 
-    train_dataset.to_disk(tmp_path / "train.jsonl", force=True)
-    train_dataset_loaded = Dataset("train").from_disk(tmp_path / "train.jsonl")
+    train_dataset.to_disk(tmp_path, force=True)
+    train_dataset_loaded = Dataset("train").from_disk(tmp_path)
     assert len(train_dataset_loaded.operations) == 0
     assert train_dataset_loaded.commit_hash == train_dataset.commit_hash
 
     train_dataset.apply_("recon.v1.upcase_labels")
 
-    train_dataset.to_disk(tmp_path / "train.jsonl", force=True)
-    train_dataset_loaded_2 = Dataset("train").from_disk(tmp_path / "train.jsonl")
+    train_dataset.to_disk(tmp_path, force=True)
+    train_dataset_loaded_2 = Dataset("train").from_disk(tmp_path)
 
     assert len(train_dataset_loaded_2.operations) == 1
     assert train_dataset_loaded_2.commit_hash == train_dataset.commit_hash
