@@ -32,3 +32,6 @@ def test_spacy_recognizer(nlp, test_texts):
     assert examples[0].text == test_texts[0]
     assert len(examples[0].spans) == 3
     assert len(examples[1].spans) == 2
+
+    scorer = recognizer.evaluate(examples, verbose=False)
+    assert scorer.ents_f == 100
